import { Agile, Integration } from '../internal';
export declare class Integrations {
    agileInstance: () => Agile;
    integrations: Set<Integration>;
    /**
     * The Integrations Class manages all Integrations for an Agile Instance
     * and provides an interface to easily update
     * and invoke functions in all registered Integrations.
     *
     * @internal
     * @param agileInstance - Instance of Agile the Integrations belongs to.
     */
    constructor(agileInstance: Agile);
    /**
     * Integrates the specified Integration into AgileTs
     * and sets it to ready when the binding was successful.
     *
     * @public
     * @param integration - Integration to be integrated into AgileTs.
     */
    integrate(integration: Integration): Promise<boolean>;
    /**
     * Updates the specified UI-Component Instance
     * with the updated data object in all registered Integrations that are ready.
     *
     * In doing so, it calls the `updateMethod()` method
     * in all registered Integrations with the specified parameters.
     *
     * @public
     * @param componentInstance - Component Instance to be updated.
     * @param updatedData - Data object with updated data.
     */
    update(componentInstance: any, updatedData: Object): void;
    /**
     * Returns a boolean indicating whether any Integration
     * has been registered with the Agile Instance or not.
     *
     * @public
     */
    hasIntegration(): boolean;
}
